/* ---------------------------------------------------------- */
/*  kome_audio_drv.c                                          */
/* ---------------------------------------------------------- */

/*------------------------------------------------------------- */
/** @file
    @brief		V850/DAC audio driver - SDL ver
    @author		rerofumi
    @since		Jul.26.2007
*/
/*-----------------------------------------------------
 Copyright (C) 2007 rerofumi <rero2@yuumu.org>
 -  Kome wo kame.
 ------------------------------------------------------*/

/*-------------------------------*/
/* include                       */
/*-------------------------------*/

#include "SDL.h"
#include "SDL_audio.h"

#include "kome_audio_drv.h"

/*-------------------------------*/
/* define                        */
/*-------------------------------*/

/*-------------------------------*/
/* struct                        */
/*-------------------------------*/

/*-------------------------------*/
/* global value                  */
/*-------------------------------*/

//! ̃obt@TCh (0 or 1)
int  audio_buffer_side;

//! o̓obt@
unsigned char  audio_buffer[KOMEAUDIO_BUFFERSIZE*2];

//! o̓`l
KAudioChannel  se_channel[KOMEAUDIO_CHANNEL];


/*-------------------------------*/
/* local routine                 */
/*-------------------------------*/

//! callback ֐
void audioDoneDMA(void *unused, Uint8 *stream, int len);

//! mixing Ȃ玟̃obt@p
void  rendering_next_buffer(void);


/*-------------------------------*/
/* interrupt vector              */
/*-------------------------------*/

/*---------------------------*/
/* implementatiion               */
/*-------------------------------*/

/*------------------------------------------------------*/
/* interface                                            */
/*------------------------------------------------------*/

/* --- komeAudioInitialize() */
/**
 * I[fBIhCȍ
 * DAC̃|[gݒ
 * 
 * @return Ȃ AUDIO_DONEAȊOŎs
 *
 */
int komeAudioInitialize(void)
{
  SDL_AudioSpec fmt;

  fmt.freq = 8000;
  fmt.format = AUDIO_U8;
  fmt.channels = 1;
  fmt.samples = KOMEAUDIO_BUFFERSIZE;
  fmt.callback = audioDoneDMA;
  fmt.userdata = NULL;

  /* I[fBIfoCXI[văTEh̍ĐJn! */
  if ( SDL_OpenAudio(&fmt, NULL) < 0 ) {
    return(AUDIO_ERROR);
  }

  return(AUDIO_DONE);
}


/* --- komeAudioStart() */
/**
 * I[fBIhCo̓X^[g
 * ^C}DMAAĊ荞݂Jn
 * 
 * @return Ȃ AUDIO_DONEAȊOŎs
 *
 */
int komeAudioStart(void)
{
  int i;

  /* --- first data */
  for(i=0; i<KOMEAUDIO_CHANNEL; i++) {
    se_channel[i].rest = 0;
    se_channel[i].repeat = 0;
  }
  audio_buffer_side = 1;
  rendering_next_buffer();
  audio_buffer_side = 0;
  rendering_next_buffer();

  /* --- callback start */
  SDL_PauseAudio(0);

  return(AUDIO_DONE);
}


/* --- komeAudioStop() */
/**
 * I[fBIhCo̓~
 * DMA̒~DAC̒~
 * 
 * @return Ȃ AUDIO_DONEAȊOŎs
 *
 */
int komeAudioStop(void)
{
  int  i;

  /* --- ~ */
  SDL_CloseAudio();

  /* --- channel stop */
  for(i=0; i<KOMEAUDIO_CHANNEL; i++) {
    se_channel[i].rest = 0;
    se_channel[i].repeat = 0;
  }
  return(AUDIO_DONE);
}


/* --- komeAudioSERequest() */
/**
 * `l̔NGXg
 * 
 * @param  ch [in] `l
 * @param  ptr [in] f[^|C^
 * @param  size [in] f[^TCY
 * @param  repeat [in] s[gtO (0=oneshot, 1=repeat)
 *
 * @return Ȃ AUDIO_DONEAȊOŎs
 *
 */
int komeAudioSERequest(int ch, const unsigned char *ptr, int size, int repeat)
{
	int  align;

  /* --- assartion */
  if ((ch < 0) || (ch > KOMEAUDIO_CHANNEL)) {
    return(AUDIO_WRONG_CHANNEL);
  }
  
  /* --- channel start */
  SDL_LockAudio();
  align = (size + 3) & 0x7ffffffc;
  se_channel[ch].size = align;
  se_channel[ch].rest = align;
  se_channel[ch].data = ptr;
  se_channel[ch].repeat = repeat;
  SDL_UnlockAudio();

  return(AUDIO_DONE);
}


/* --- komeAudioSEStop() */
/**
 * `l̔~
 * 
 * @param  ch [in] ~`l
 *
 * @return Ȃ AUDIO_DONEAȊOŎs
 *
 */
int komeAudioSEStop(int ch)
{
  /* --- assartion */
  if ((ch < 0) || (ch > KOMEAUDIO_CHANNEL)) {
    return(AUDIO_WRONG_CHANNEL);
  }
  /* --- channel stop */
  SDL_LockAudio();
  se_channel[ch].rest = 0;
  se_channel[ch].repeat = 0;
  SDL_UnlockAudio();

  return(AUDIO_DONE);
}



/*------------------------------------------------------*/
/* Interrupt                                            */
/*------------------------------------------------------*/

/* --- DMA]̊荞ݏ     */
/*     DMA]vƃobt@̗p */
void audioDoneDMA(void *unused, Uint8 *stream, int len)
{
  unsigned char  *buffer;

  /* swap buffer */
  if (audio_buffer_side == 0) {
    audio_buffer_side = 1;
    buffer = audio_buffer + KOMEAUDIO_BUFFERSIZE;
  }
  else {
    audio_buffer_side = 0;
    buffer = audio_buffer;
  }

  /* --- obt@]Jn */
  SDL_MixAudio(stream, buffer, KOMEAUDIO_BUFFERSIZE, SDL_MIX_MAXVOLUME);

  /* --- ̃obt@f[^~LVO */
  rendering_next_buffer();
}


/*------------------------------------------------------*/
/* Local routine                                        */
/*------------------------------------------------------*/

/* --- DMA]ĂԂɎ̃obt@pӂĂ */
void  rendering_next_buffer(void)
{
  int  i, ch;
  int  buf_ptr;
  int  tmp_buffer[KOMEAUDIO_BUFFERSIZE];
  int  wav, flag;
  unsigned char  byte;

  /* --- obt@NA */
  if (audio_buffer_side == 0) {
    buf_ptr = KOMEAUDIO_BUFFERSIZE;
  }
  else {
    buf_ptr = 0;
  }
  for(i=0; i<KOMEAUDIO_BUFFERSIZE; i++) {
    tmp_buffer[i] = 0;
  }

  /* --- `l~LVO */
  flag = 0;
  for(ch=0; ch<KOMEAUDIO_CHANNEL; ch++) {
    /* - łȂΎ */
    if (se_channel[ch].rest <= 0) continue;
    flag = 1;
    /* - ~LVO */
    for(i=0; i<KOMEAUDIO_BUFFERSIZE; i++) {
      byte = *(se_channel[ch].data + (se_channel[ch].size - se_channel[ch].rest));
      wav = ((int)byte) - 128;
      tmp_buffer[i] += wav;
      se_channel[ch].rest -= 1;
      if (se_channel[ch].rest <= 0) {
        if (se_channel[ch].repeat == 0) {
          break;
        }
        else {
          se_channel[ch].rest = se_channel[ch].size;
        }
      }
    }
  }

  /* --- ʎq */
  if (flag == 0) {
    for(i=0; i<KOMEAUDIO_BUFFERSIZE; i++) {
       audio_buffer[buf_ptr+i] = 0x80;
    }
  }
  else {
    for(i=0; i<KOMEAUDIO_BUFFERSIZE; i++) {
      wav = tmp_buffer[i] + 128;
      if (wav > 255) {
        wav = 255;
      }
      if (wav < 0) {
        wav = 0;
      }
      audio_buffer[buf_ptr+i] = (unsigned char)wav;
    }
  }
}
