/* ---------------------------------------------------------- */
/*  kv_display.c                                              */
/* ---------------------------------------------------------- */

/*------------------------------------------------------------- */
/** @file
    @brief		KomeVader graphic display manager
    @author		rerofumi
    @since		Aug.04.2007
*/
/*-----------------------------------------------------
 Copyright (C) 2007 rerofumi <rero2@yuumu.org>
 -  Kome wo kame.
 ------------------------------------------------------*/

/*-------------------------------*/
/* include                       */
/*-------------------------------*/

#include "kome_display_drv.h"
#include "kv_display.h"


/* --- p^[f[^ */
#include "graphic/game_chr.h"


/*-------------------------------*/
/* define                        */
/*-------------------------------*/

/*-------------------------------*/
/* struct                        */
/*-------------------------------*/

/*-------------------------------*/
/* global value                  */
/*-------------------------------*/

/*-------------------------------*/
/* local routine                 */
/*-------------------------------*/

//! XRAp10i\
void  disp_decimal6(int num, int x, int y);

/*-------------------------------*/
/* interrupt vector              */
/*-------------------------------*/

/*-------------------------------*/
/* implementatiion               */
/*-------------------------------*/

/*------------------------------------------------------*/
/* interface                                            */
/*------------------------------------------------------*/

/* --- kvDisplayVSync() */
/**
 * VSync wait \]
 * 
 */
void kvDisplayVSync(void)
{
  komeDisplayVSync();
}


/* --- kvDisplayGameStart() */
/**
 * Q[X^[gSʏ
 * 
 * @param hiscore [in] nCXRA
 * @param ship [in] c@\
 */
void kvDisplayGameStart(int hiscore, int ship)
{
  komeDisplaySwCrtc(0);
  komeDisplayClearScreen();
  komeDisplayFill(  1, 224, 240-1,  16, 0xff);
  komeDisplayFill(240,   0,   2, 240, 0xff);
  komeDisplayString(248, 200, "KOME-");
  komeDisplayString(264, 208, "VADER");
  komeDisplayString(248,  32, "SCORE");
  komeDisplayString(248,  56, "HISCORE");
  komeDisplayString(248,  80, "REST");
  komeDisplayASync();
  kvDisplayScore(256, 40, 0);
  kvDisplayHiscore(256, 64, hiscore);
  kvDisplayMyshipRest(ship);
  komeDisplayASync();
  komeDisplaySwCrtc(1);
}


/* --- kvDisplayScore() */
/**
 * XRA\NGXg
 * 
 * @param x [in] \ʒu x
 * @param y [in] \ʒu y
 * @param score [in] XRA
 */
void kvDisplayScore(int x, int y, int score)
{
  disp_decimal6(score, x, y);
}


/* --- kvDisplayHiscore() */
/**
 * XRA\NGXg
 * 
 * @param x [in] \ʒu x
 * @param y [in] \ʒu y
 * @param hiscore [in] nCXRA
 */
void kvDisplayHiscore(int x, int y, int hiscore)
{
  disp_decimal6(hiscore, x, y);
}


/* --- kvDisplayMyshipRest() */
/**
 * @c\NGXg
 * 
 * @param rest [in] c@
 */
void kvDisplayMyshipRest(int rest)
{
  int i;

  komeDisplayFont(256, 88, '0'+rest);
  for(i=0; i<3; i++) {
    if (i < rest) {
      kvDisplayMyship(256+(i*14), 96, DISPLAY_MYSHIP_NORMAL);
    }
    else {
      kvDisplayMyship(256+(i*14), 96, DISPLAY_MYSHIP_ERASE);
    }
  }
  
}


/* --- kvDisplayShield() */
/**
 * g[`J\NGXg
 * 
 * @param x [in] \ʒu x
 * @param y [in] \ʒu y
 * @param pattern [in] \pattern
 */
void kvDisplayShield(int x, int y, int pattern)
{
  if (pattern == DISPLAY_SHIELD_ERASE) {
    komeDisplayFill(x, y, 4, 8, 0);
  }
  else if (pattern == DISPLAY_SHIELD_NORMAL) {
    komeDisplayPattern(x, y, 4, 8, shield_1);
  }
  else if (pattern == DISPLAY_SHIELD_UP) {
    komeDisplayPattern(x, y, 4, 8, shield_2);
  }
  else if (pattern == DISPLAY_SHIELD_DOWN) {
    komeDisplayPattern(x, y, 4, 8, shield_3);
  }
  else if (pattern == DISPLAY_SHIELD_LEFT) {
    komeDisplayPattern(x, y, 4, 8, shield_4);
  }
  else if (pattern == DISPLAY_SHIELD_RIGHT) {
    komeDisplayPattern(x, y, 4, 8, shield_5);
  }
}


/* --- kvDisplayMyship() */
/**
 * @\NGXg
 * 
 * @param x [in] \ʒu x
 * @param y [in] \ʒu y
 * @param pattern [in] \pattern
 */
void kvDisplayMyship(int x, int y, int pattern)
{
  if (pattern == DISPLAY_MYSHIP_ERASE) {
    komeDisplayFill(x, y, 13, 8, 0);
  }
  else if (pattern == DISPLAY_MYSHIP_NORMAL) {
    komeDisplayPattern(x, y, 13, 8, myship);
  }
  else if (pattern == DISPLAY_MYSHIP_MISS_1) {
    komeDisplayPattern(x, y, 13, 8, miss_0);
  }
  else if (pattern == DISPLAY_MYSHIP_MISS_2) {
    komeDisplayPattern(x, y, 13, 8, miss_1);
  }
}


/* --- kvDisplayShot() */
/**
 * e\NGXg
 * 
 * @param x [in] \ʒu x
 * @param y [in] \ʒu y
 * @param pattern [in] \pattern
 */
void kvDisplayShot(int x, int y, int pattern)
{
  if (pattern == DISPLAY_SHOT_ERASE) {
    komeDisplayFill(x, y, 3, 8, 0);
  }
  else if (pattern == DISPLAY_SHOT_MYSHIP) {
    komeDisplayPattern(x, y, 3, 8, shot);
  }
  else if (pattern == DISPLAY_SHOT_ENEMY_1_1) {
    komeDisplayPattern(x, y, 3, 8, attack1_0);
  }
  else if (pattern == DISPLAY_SHOT_ENEMY_1_2) {
    komeDisplayPattern(x, y, 3, 8, attack1_1);
  }
  else if (pattern == DISPLAY_SHOT_ENEMY_2_1) {
    komeDisplayPattern(x, y, 3, 8, attack2_0);
  }
  else if (pattern == DISPLAY_SHOT_ENEMY_2_2) {
    komeDisplayPattern(x, y, 3, 8, attack1_1);
  }
}


/* --- kvDisplayEnemy() */
/**
 * G\NGXg
 * 
 * @param x [in] \ʒu x
 * @param y [in] \ʒu y
 * @param pattern [in] \pattern
 */
void kvDisplayEnemy(int x, int y, int pattern)
{
  if (pattern == DISPLAY_ENEMY_10_ERASE) {
    komeDisplayFill(x, y, 12, 8, 0);
  }
  else if (pattern == DISPLAY_ENEMY_10_1) {
    komeDisplayPattern(x, y, 12, 8, inv1_0);
  }
  else if (pattern == DISPLAY_ENEMY_10_2) {
    komeDisplayPattern(x, y, 12, 8, inv1_1);
  }
  else if (pattern == DISPLAY_ENEMY_20_ERASE) {
    komeDisplayFill(x, y, 10, 8, 0);
  }
  else if (pattern == DISPLAY_ENEMY_20_1) {
    komeDisplayPattern(x, y, 10, 8, inv2_0);
  }
  else if (pattern == DISPLAY_ENEMY_20_2) {
    komeDisplayPattern(x, y, 10, 8, inv2_1);
  }
  else if (pattern == DISPLAY_ENEMY_30_ERASE) {
    komeDisplayFill(x, y, 8, 8, 0);
  }
  else if (pattern == DISPLAY_ENEMY_30_1) {
    komeDisplayPattern(x, y, 8, 8, inv3_0);
  }
  else if (pattern == DISPLAY_ENEMY_30_2) {
    komeDisplayPattern(x, y, 8, 8, inv3_1);
  }
  else if (pattern == DISPLAY_ENEMY_HIT_ERASE) {
    komeDisplayFill(x, y, 8, 8, 0);
  }
  else if (pattern == DISPLAY_ENEMY_HIT) {
    komeDisplayPattern(x, y, 8, 8, hit);
  }
}


/* --- kvDisplayMystery() */
/**
 * ~Xe[\NGXg
 * 
 * @param x [in] \ʒu x
 * @param y [in] \ʒu y
 * @param pattern [in] \pattern
 */
void kvDisplayMystery(int x, int y, int pattern)
{
  if (pattern == DISPLAY_MYSTERY_ERASE) {
    komeDisplayFill(x, y, 16, 8, 0);
  }
  else if (pattern == DISPLAY_MYSTERY_1) {
    komeDisplayPattern(x, y, 16, 8, mistery_0);
  }
  else if (pattern == DISPLAY_MYSTERY_2) {
    komeDisplayPattern(x, y, 16, 8, mistery_1);
  }
  else if (pattern == DISPLAY_MYSTERY_HIT_ERASE) {
    komeDisplayFill(x, y, 8, 8, 0);
  }
  else if (pattern == DISPLAY_MYSTERY_HIT) {
    komeDisplayPattern(x, y, 8, 8, hit);
  }
}


/*------------------------------------------------------*/
/* local routine                                        */
/*------------------------------------------------------*/

/* -------------------------------------- */
/* --- XRAp10i\(60) */
void  disp_decimal6(int num, int x, int y)
{
  int i;
  int rest, k, base;
  char  c;

  base = 100000;
  rest = num;
  for(i=0; i<6; i++) {
    k = (rest / base) % 10;
    rest = (rest % base);
    base /= 10;
    c = '0' + (char)k;
    komeDisplayFont(x, y, c);
    x += 8;
  }
}

