/* ---------------------------------------------------------- */
/*  kv_enemy.c                                                */
/* ---------------------------------------------------------- */

/*------------------------------------------------------------- */
/** @file
    @brief		KomeVader enemy comvoy manage
    @author		rerofumi
    @since		Aug.05.2007
*/
/*-----------------------------------------------------
 Copyright (C) 2007 rerofumi <rero2@yuumu.org>
 -  Kome wo kame.
 ------------------------------------------------------*/

/*-------------------------------*/
/* include                       */
/*-------------------------------*/

#include "kv_display.h"
#include "kv_sound.h"
#include "kv_shot.h"
#include "kv_myship.h"
#include "kv_enemy.h"

/*-------------------------------*/
/* define                        */
/*-------------------------------*/

#define ENEMY_WALK_DOWN 0x02

/*-------------------------------*/
/* struct                        */
/*-------------------------------*/

//! GIuWFNg\
typedef struct {
  short  pos_x, pos_y;
  char   flag;
} KVEnemy, PKVEnemy;

enum {
  ENEMY_NONE = 0,
  ENEMY_TYPE_10,
  ENEMY_TYPE_20,
  ENEMY_TYPE_30,
  ENEMY_TYPE_HIT,
};

/*-------------------------------*/
/* global value                  */
/*-------------------------------*/

//! GU
KVEnemy  m_enemy[6*ENEMY_WIDTH];

//! GspJE^
int  m_enemy_walk_counter;

//! ړG̔ԍ
int  m_enemy_ptr;

//! sp^C}
int  m_enemy_timer;

//! sNGXgtO
int  m_enemy_walk_request;

//! sJE^
int  m_enemy_walk_beep_counter;

//! Gړ
int  m_enemy_dir;

//! U󂯂GID
int  m_enemy_hit;

//! qbgꎞ~JE^
int  m_enemy_hit_stop;

//! UԊupEFCgJE^
int  m_enemy_attack_timing;



/*-------------------------------*/
/* local routine                 */
/*-------------------------------*/

//! ]mF
int  enemy_check_turn(void);

//! s̏
void enemy_walk_se(void);

//! UNGXg
void enemy_attack(void);


/*-------------------------------*/
/* interrupt vector              */
/*-------------------------------*/

/*-------------------------------*/
/* implementatiion               */
/*-------------------------------*/

/*------------------------------------------------------*/
/* interface                                            */
/*------------------------------------------------------*/

/* --- kvEnemyInitialize() */
/**
 * G
 * 
 * @param stage [in] ݂̃Xe[W(Փxݒp)
 */
void kvEnemyInitialize(int stage)
{
  int  i, j, c;
  int  h, type;

  /* --- ̍(Փx) */
  h = ENEMY_HEIGHT + (stage * 32);
  if (h > 168) {
    h = 168;
  }

  /* --- R{Cݒ */
  for(j=0; j<6; j++) {
    type = (j / 2) + 1;
    for(i=0; i<ENEMY_WIDTH; i++) {
      c = i + (j * ENEMY_WIDTH);
      m_enemy[c].pos_x = 16 + (i * 16);
      m_enemy[c].pos_y = h;
      m_enemy[c].flag = type;
    }
    h -= 12;
  }

  /* --- JE^ */
  m_enemy_ptr = 0;
  m_enemy_walk_counter = 0;
  m_enemy_dir = 0;
  m_enemy_walk_request = 0;
  m_enemy_walk_beep_counter = 0;
  m_enemy_hit_stop = 0;
  m_enemy_attack_timing = 0;
}


/* --- kvEnemyTick() */
/**
 * Gړ
 * 
 * @return ENEMY_DONEŒʏIAENEMY_INVADEŐNQ[I[o
 */
int kvEnemyTick(void)
{
  int  i, id, walk, invade;
  int  post_x, post_y;
  int  pat;

  m_enemy_timer += 1;

  /* --- GȂ */
  if (m_enemy_ptr == -1) {
    return(ENEMY_DONE);
  }

  /* --- qbg}[N\̂߂Ɉꎞ~ */
  if (m_enemy_hit_stop > 0) {
    kvDisplayEnemy(m_enemy[m_enemy_hit].pos_x-6,
		   m_enemy[m_enemy_hit].pos_y,
		   DISPLAY_ENEMY_10_ERASE);
    m_enemy_hit_stop -= 1;
    if (m_enemy_hit_stop > 0) {
      kvDisplayEnemy(m_enemy[m_enemy_hit].pos_x-4,
		     m_enemy[m_enemy_hit].pos_y,
		     DISPLAY_ENEMY_HIT);
    }
    else {
      /* --- ꂽG͏ */
      m_enemy[m_enemy_hit].flag = 0;
    }
    return(ENEMY_DONE);
  }

  /* --- ړ */
  invade = 0;
  if (m_enemy[m_enemy_ptr].flag != 0) {
    post_x = m_enemy[m_enemy_ptr].pos_x;
    post_y = m_enemy[m_enemy_ptr].pos_y;
    if ((m_enemy_dir & ENEMY_WALK_DOWN) != 0) {
      m_enemy[m_enemy_ptr].pos_y += 8;
      if (m_enemy[m_enemy_ptr].pos_y >= 208) {
	invade = 1;
      }
    }
    else {
      if ((m_enemy_dir & 0x01) == 0) {
	/* E */
	m_enemy[m_enemy_ptr].pos_x += 4;
      }
      else {
	/*  */
	m_enemy[m_enemy_ptr].pos_x -= 4;
      }
    }
    /* g[`JƐڐG */
    kvMyshipShieldCheckHit(m_enemy[m_enemy_ptr].pos_x-4,
			   m_enemy[m_enemy_ptr].pos_y+8);
    kvMyshipShieldCheckHit(m_enemy[m_enemy_ptr].pos_x,
			   m_enemy[m_enemy_ptr].pos_y+8);
    kvMyshipShieldCheckHit(m_enemy[m_enemy_ptr].pos_x+4,
			   m_enemy[m_enemy_ptr].pos_y+8);

    /* --- \ */
    switch(m_enemy[m_enemy_ptr].flag) {
    case ENEMY_TYPE_10:
      pat = DISPLAY_ENEMY_10_1 + (m_enemy_walk_counter & 1);
      kvDisplayEnemy(post_x-6, post_y, DISPLAY_ENEMY_10_ERASE);
      kvDisplayEnemy(m_enemy[m_enemy_ptr].pos_x-6,
		     m_enemy[m_enemy_ptr].pos_y,
		     pat);
      break;
      
    case ENEMY_TYPE_20:
      kvDisplayEnemy(post_x-5, post_y, DISPLAY_ENEMY_20_ERASE);
      kvDisplayEnemy(m_enemy[m_enemy_ptr].pos_x-5,
		     m_enemy[m_enemy_ptr].pos_y,
		     DISPLAY_ENEMY_20_1 + (m_enemy_walk_counter & 1));
      break;
      
    case ENEMY_TYPE_30:
      kvDisplayEnemy(post_x-4, post_y, DISPLAY_ENEMY_30_ERASE);
      kvDisplayEnemy(m_enemy[m_enemy_ptr].pos_x-4,
		     m_enemy[m_enemy_ptr].pos_y,
		     DISPLAY_ENEMY_30_1 + (m_enemy_walk_counter & 1));
      break;
    }
  }

  /* --- ɕG */
  walk = 0;
  for(i=0; i<(ENEMY_WIDTH*6); i++) {
    id = (m_enemy_ptr + 1 + i) % (ENEMY_WIDTH*6);
    if (id == 0) {
      /* ꏄ */
      m_enemy_walk_request = 1;
      m_enemy_walk_counter += 1;
      m_enemy_dir = m_enemy_dir & 0x01;
      enemy_check_turn();
    }
    if (m_enemy[id].flag != 0) {
      walk = 1;
      break;
    }
  }
  if (walk == 0) {
    m_enemy_ptr = -1;
  }
  else {
    m_enemy_ptr = id;
  }

  /* --- U */
  enemy_attack();

  /* --- sǗ */
  enemy_walk_se();

  if (invade != 0) {
    return(ENEMY_INVADE);
  }
  return(ENEMY_DONE);
}


/* --- kvEnemyGetHeight() */
/**
 * G̍ŉʒu
 * 
 * @return ĂGōłɂzYW
 */
int kvEnemyGetHeight(void)
{
  int  i, height;

  height = 0;
  for(i=0; i<(ENEMY_WIDTH*6); i++) {
    if (m_enemy[i].flag != 0) {
      height = m_enemy[i].pos_y;
      break;
    }
  }
  return(height);
}


/* --- kvEnemyGetNum() */
/**
 * G̐
 * 
 * @return ĂG̐
 */
int kvEnemyGetNum(void)
{
  int  i, count;

  count = 0;
  for(i=0; i<(ENEMY_WIDTH*6); i++) {
    if (m_enemy[i].flag != 0) {
      count += 1;
    }
  }
  return(count);
}


/* --- kvEnemyCheckHit() */
/**
 * GƒȅՓ˔
 * 
 * @return ENEMY_DONEŔ薳, ENEMY_HIT_** ł
 */
int kvEnemyCheckHit(int x, int y)
{
  int lower, i;
  int  hit;

  hit = 0;
  lower = kvEnemyGetHeight();
  if ((lower+8) > y) {
    for(i=0; i<(ENEMY_WIDTH*6); i++) {
      if (m_enemy[i].flag == 0) continue;
      if ((m_enemy[i].pos_y > y) ||
	  (m_enemy[i].pos_y+8 < y)) continue;
      /* -- Փ˔ */
      if (m_enemy[i].flag == ENEMY_TYPE_10) {
	if ((m_enemy[i].pos_x-6 <= x) &&
	    (m_enemy[i].pos_x+6 > x)) {
	  hit = ENEMY_HIT_10;
	}
      }
      else if(m_enemy[i].flag == ENEMY_TYPE_20) {
	if ((m_enemy[i].pos_x-5 <= x) &&
	    (m_enemy[i].pos_x+5 > x)) {
	  hit = ENEMY_HIT_20;
	}
      }
      else if(m_enemy[i].flag == ENEMY_TYPE_30) {
	if ((m_enemy[i].pos_x-4 <= x) &&
	    (m_enemy[i].pos_x+4 > x)) {
	  hit = ENEMY_HIT_30;
	}
      }
      /* -- 肾 */
      if (hit != 0) {
	/* - G̔ԍTĈꎞ~JE^Zbg */
	m_enemy_hit = i;
	m_enemy_hit_stop = 20;
	break;
      }
    }
  }

  return(hit);
}



/*------------------------------------------------------*/
/* local routine                                        */
/*------------------------------------------------------*/

/* -------------------------------- */
/* --- ]mF */
int  enemy_check_turn(void)
{
  int  i;
  int  left, right;

  /* E̒[mF */
  left = right = -1;
  for(i=0; i<(ENEMY_WIDTH*6); i++) {
    if (m_enemy[i].flag != 0) {
      if (left == -1) {
	left = m_enemy[i].pos_x;
      }
      else {
	if (left > m_enemy[i].pos_x) {
	  left = m_enemy[i].pos_x;
	}
      }
      if (right == -1) {
	right = m_enemy[i].pos_x;
      }
      else {
	if (right < m_enemy[i].pos_x) {
	  right = m_enemy[i].pos_x;
	}
      }
    }
  }

  /* ---  ismF */
  if ((m_enemy_dir & 0x01) != 0) {
    /* O܂ōɈړĂ */
    if (left < 16) {
      m_enemy_dir = 0x00 | ENEMY_WALK_DOWN;
    }
  }
  else {
    /* O܂ŉEɈړĂ */
    if (right > 224) {
      m_enemy_dir = 0x01 | ENEMY_WALK_DOWN;
    }
  }

  return(m_enemy_dir);
}


/* -------------------------------- */
/* --- s̏ */
void enemy_walk_se(void)
{
  int  se[4] = { SE_WALK1, SE_WALK2, SE_WALK3, SE_WALK4 };

  if (m_enemy_walk_request != 0) {
    if (m_enemy_timer > 24) {
      kvSoundRequest(2, se[m_enemy_walk_beep_counter]);
      m_enemy_walk_beep_counter = (m_enemy_walk_beep_counter + 1) % 4;
      m_enemy_walk_request = 0;
      m_enemy_timer = 0;
    } 
  }
}


/* -------------------------------- */
/* --- UNGXg */
void enemy_attack(void)
{
  int  num;
  int  i, j;
  int  pos, shot;

  /* --- ˃^C~O҂ */
  m_enemy_attack_timing += 1;
  num = kvEnemyGetNum();
  num = num + 15;
  if (m_enemy_attack_timing > num) {
    m_enemy_attack_timing = 0;
    /* - ǂ̓G˂邩 */
    shot = 0;
    pos = (m_enemy_walk_counter ^ 0xa5) % ENEMY_WIDTH;
    for(i=0; i<10; i++) {
      for(j=0; j<6; j++) {
	if (m_enemy[pos+(ENEMY_WIDTH*j)].flag != 0) {
	  kvShotEnemyshotRequest(m_enemy[pos+(ENEMY_WIDTH*j)].pos_x,
				 m_enemy[pos+(ENEMY_WIDTH*j)].pos_y+8);
	  shot = 1;
	  break;
	}
      }
      if (shot == 1) break;
      pos = (pos + 1) % 10;
    }
  }
}

