/* ---------------------------------------------------------- */
/*  kv_mystery.c                                              */
/* ---------------------------------------------------------- */

/*------------------------------------------------------------- */
/** @file
    @brief		KomeVader mystery manage
    @author		rerofumi
    @since		Aug.05.2007
*/
/*-----------------------------------------------------
 Copyright (C) 2007 rerofumi <rero2@yuumu.org>
 -  Kome wo kame.
 ------------------------------------------------------*/

/*-------------------------------*/
/* include                       */
/*-------------------------------*/

#include "kv_display.h"
#include "kv_sound.h"
#include "kv_mystery.h"

/*-------------------------------*/
/* define                        */
/*-------------------------------*/

#define MYSTERY_Y  16

/*-------------------------------*/
/* struct                        */
/*-------------------------------*/

/*-------------------------------*/
/* global value                  */
/*-------------------------------*/

//! ʒu
int  m_mystery_x, m_mystery_y;

//! ړ
int  m_mystery_dir;

//! ړpJE^
int  m_mystery_walk_counter;

//! oJE^
int  m_mystery_appear_counter;

//! pJE^
int  m_mystery_table_counter;

//! qbg̃EFCg
int  m_mystery_hit_wait;

//! _e[u
const int  m_mystery_table[] = {
  50, 100, 50, 100, 200, 300,
};

/*-------------------------------*/
/* local routine                 */
/*-------------------------------*/

/*-------------------------------*/
/* interrupt vector              */
/*-------------------------------*/

/*-------------------------------*/
/* implementatiion               */
/*-------------------------------*/

/*------------------------------------------------------*/
/* interface                                            */
/*------------------------------------------------------*/

/* --- kvMysteryInitialize() */
/**
 * {[iX
 * 
 */
void kvMysteryInitialize(void)
{
  m_mystery_hit_wait = 0;
  m_mystery_table_counter = 0;
  m_mystery_appear_counter = 240;
}


/* --- kvMysteryTick() */
/**
 * {[iXړ
 * 
 * @return MYSTERY_HITŎ@e, MYSTERY_DONEŒʏI
 */
int kvMysteryTick(void)
{
  int  hit;

  hit = 0;

  /* --- qbg̃}[Nƒ~ */
  if (m_mystery_hit_wait > 0) {
    m_mystery_hit_wait -= 1;
    if (m_mystery_hit_wait == 0) {
      kvDisplayMystery(m_mystery_x-4,
		       m_mystery_y,
		       DISPLAY_MYSTERY_HIT_ERASE);
      m_mystery_appear_counter = m_mystery_table[(m_mystery_table_counter%6)] * 5;
      m_mystery_walk_counter = 0;
      m_mystery_x = -32;
    }
    return(MYSTERY_DONE);
  }

  /* --- o܂ł̑҂ */
  if (m_mystery_appear_counter > 0) {
    m_mystery_appear_counter -= 1;
    if (m_mystery_appear_counter == 0) {
      /* - oꏈ */
      m_mystery_walk_counter = (240 - 16);
      m_mystery_dir = m_mystery_table_counter & 1;
      if (m_mystery_dir == 0) {
	m_mystery_x = 8;
	m_mystery_y = MYSTERY_Y;
      }
      else {
	m_mystery_x = 240 - 8;
	m_mystery_y = MYSTERY_Y;
      }
    }
  }

  /* --- ړ */
  if (m_mystery_walk_counter > 0) {
    /* s */
    if ((m_mystery_walk_counter % 25) == 0) {
      kvSoundRequest(3, SE_MYSTERY_APPEAR);
    }
    /*  */
    kvDisplayMystery(m_mystery_x-8,
		     m_mystery_y,
		     DISPLAY_MYSTERY_ERASE);
    /* ړ */
    if (m_mystery_dir == 0) {
      m_mystery_x += 1;
    }
    else {
      m_mystery_x -= 1;
    }
    /* JEg */
    m_mystery_walk_counter -= 1;
    if (m_mystery_walk_counter == 0) {
      m_mystery_appear_counter = m_mystery_table[(m_mystery_table_counter%6)] * 5;
      hit = 1;
    }
    /* \ */
    if (hit == 0) {
      kvDisplayMystery(m_mystery_x-8,
		       m_mystery_y,
		       DISPLAY_MYSTERY_1 + ((m_mystery_walk_counter>>2) & 1));
    }
    else {
      m_mystery_x = -32;
    }
  }

  return(MYSTERY_DONE);
}


/* --- kvMysteryCheckHit() */
/**
 * {[iXƒe̓蔻
 * 
 * @param x [in] mFWX
 * @param y [in] mFWY
 * @return MYSTERY_HITŎ@e, MYSTERY_DONEł͂
 */
int kvMysteryCheckHit(int x, int y)
{
  /* --- ݃qbg}[N\̂ߖ */
  if (m_mystery_hit_wait > 0) {
    return(MYSTERY_DONE);
  }

  if ((y >= MYSTERY_Y) && (y < (MYSTERY_Y+8))) {
    if ((x >= (m_mystery_x-8)) && (x < (m_mystery_x+8))) {
      
      m_mystery_hit_wait = 90;
      kvDisplayMystery(m_mystery_x-8,
		       m_mystery_y,
		       DISPLAY_MYSTERY_ERASE);
      kvDisplayMystery(m_mystery_x-4,
		       m_mystery_y,
		       DISPLAY_MYSTERY_HIT);
      return(MYSTERY_HIT);
    }
  }

  return(MYSTERY_DONE);
}


/* --- kvMysteryCheckHit() */
/**
 * {[iX̓_
 * 
 * @return {[iX̓_
 */
int kvMysteryGetScore(void)
{
  return(m_mystery_table[(m_mystery_table_counter%6)]);
}


/* --- kvMysteryCheckHit() */
/**
 * {[iX̓_pJE^
 * 
 * @return {[iX̓_
 */
void kvMysteryAddCount(void)
{
  m_mystery_table_counter += 1;
}

